@@ -2,12 +2,13 @@
 
 from synthesis.synthesizers.marginal import MarginalSynthesizer, UniformSynthesizer
 from synthesis.synthesizers.contingency import ContingencySynthesizer, ContingencySynthesizerFix
-from synthesis.synthesizers.privbayes import PrivBayes
+from synthesis.synthesizers.privbayes import PrivBayes, PrivBayesFix
 
 __all__ = [
     'MarginalSynthesizer',
     'UniformSynthesizer',
     'ContingencySynthesizer',
     'ContingencySynthesizerFix',
-    'PrivBayes'
+    'PrivBayes',
+    'PrivBayesFix'
 ]
\ No newline at end of file
