@@ -1,15 +1,12 @@
 """Transformer for preprocessing input data and postprocessing output data"""
 
 from synthesis.transformers.generalization import GeneralizeContinuous, GeneralizeCategorical, GeneralizeSchematic
-from synthesis.transformers.suppression import RemoveColumns, KeepFirstIdentifier, ReplaceUniqueValues
 from synthesis.transformers.sequence import GeneralizeDateSequence
+from synthesis.transformers.deidentification import *
 
 __all__ = [
     'GeneralizeContinuous',
     'GeneralizeCategorical',
     'GeneralizeSchematic',
-    'RemoveColumns',
-    'KeepFirstIdentifier',
-    'ReplaceUniqueValues',
     'GeneralizeDateSequence'
 ]
